"""tests/test_directives.py.

Tests to ensure that directives interact in the anticipated manner

Copyright (C) 2016 Timothy Edmund Crosley

Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated
documentation files (the "Software"), to deal in the Software without restriction, including without limitation
the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and
to permit persons to whom the Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or
substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.

"""
from base64 import b64encode

import pytest

import hug

api = hug.API(__name__)

# Fix flake8 undefined names (F821)
__hug__ = __hug__  # noqa


def test_timer():
    """Tests that the timer directive outputs the correct format, and automatically attaches itself to an API"""
    timer = hug.directives.Timer()
    assert isinstance(timer.start, float)
    assert isinstance(float(timer), float)
    assert isinstance(int(timer), int)

    timer = hug.directives.Timer(3)
    assert isinstance(timer.start, float)
    assert isinstance(float(timer), float)
    assert isinstance(int(timer), int)
    assert isinstance(str(timer), str)
    assert isinstance(repr(timer), str)
    assert float(timer) < timer.start

    @hug.get()
    @hug.local()
    def timer_tester(hug_timer):
        return hug_timer

    assert isinstance(hug.test.get(api, 'timer_tester').data, float)
    assert isinstance(timer_tester(), hug.directives.Timer)


def test_module():
    """Test to ensure the module directive automatically includes the current API's module"""
    @hug.get()
    def module_tester(hug_module):
        return hug_module.__name__

    assert hug.test.get(api, 'module_tester').data == api.module.__name__


def test_api():
    """Ensure the api correctly gets passed onto a hug API function based on a directive"""
    @hug.get()
    def api_tester(hug_api):
        return hug_api == api

    assert hug.test.get(api, 'api_tester').data is True


def test_documentation():
    """Test documentation directive"""
    assert 'handlers' in hug.directives.documentation(api=api)


def test_api_version():
    """Ensure that it's possible to get the current version of an API based on a directive"""
    @hug.get(versions=1)
    def version_tester(hug_api_version):
        return hug_api_version

    assert hug.test.get(api, 'v1/version_tester').data == 1


def test_current_api():
    """Ensure that it's possible to retrieve methods from the same version of the API"""
    @hug.get(versions=1)
    def first_method():
        return "Success"

    @hug.get(versions=1)
    def version_call_tester(hug_current_api):
        return hug_current_api.first_method()

    assert hug.test.get(api, 'v1/version_call_tester').data == 'Success'

    @hug.get()
    def second_method():
        return "Unversioned"

    @hug.get(versions=2)  # noqa
    def version_call_tester(hug_current_api):
        return hug_current_api.second_method()

    assert hug.test.get(api, 'v2/version_call_tester').data == 'Unversioned'

    @hug.get(versions=3)  # noqa
    def version_call_tester(hug_current_api):
        return hug_current_api.first_method()

    with pytest.raises(AttributeError):
        hug.test.get(api, 'v3/version_call_tester').data


def test_user():
    """Ensure that it's possible to get the current authenticated user based on a directive"""
    user = 'test_user'
    password = 'super_secret'

    @hug.get(requires=hug.authentication.basic(hug.authentication.verify(user, password)))
    def authenticated_hello(hug_user):
        return hug_user

    token = b64encode('{0}:{1}'.format(user, password).encode('utf8')).decode('utf8')
    assert hug.test.get(api, 'authenticated_hello', headers={'Authorization': 'Basic {0}'.format(token)}).data == user


def test_session_directive():
    """Ensure that it's possible to retrieve the session withing a request using the built-in session directive"""
    @hug.request_middleware()
    def add_session(request, response):
        request.context['session'] = {'test': 'data'}

    @hug.local()
    @hug.get()
    def session_data(hug_session):
        return hug_session

    assert session_data() is None
    assert hug.test.get(api, 'session_data').data == {'test': 'data'}


def test_named_directives():
    """Ensure that it's possible to attach directives to named parameters"""
    @hug.get()
    def test(time: hug.directives.Timer=3):
        return time

    assert isinstance(test(1), int)

    test = hug.local()(test)
    assert isinstance(test(), hug.directives.Timer)


def test_local_named_directives():
    """Ensure that it's possible to attach directives to local function calling"""
    @hug.local()
    def test(time: __hug__.directive('timer')=3):
        return time

    assert isinstance(test(), hug.directives.Timer)

    @hug.local(directives=False)
    def test(time: __hug__.directive('timer')=3):
        return time

    assert isinstance(test(3), int)


def test_named_directives_by_name():
    """Ensure that it's possible to attach directives to named parameters using only the name of the directive"""
    @hug.get()
    @hug.local()
    def test(time: __hug__.directive('timer')=3):
        return time

    assert isinstance(test(), hug.directives.Timer)


def test_per_api_directives():
    """Test to ensure it's easy to define a directive within an API"""
    @hug.directive(apply_globally=False)
    def test(default=None, **kwargs):
        return default

    @hug.get()
    def my_api_method(hug_test='heyyy'):
        return hug_test

    assert hug.test.get(api, 'my_api_method').data == 'heyyy'


def test_user_directives():
    """Test the user directives functionality, to ensure it will provide the set user object"""
    @hug.get()  # noqa
    def try_user(user: hug.directives.user):
        return user

    assert hug.test.get(api, 'try_user').data is None

    @hug.get(requires=hug.authentication.basic(hug.authentication.verify('Tim', 'Custom password')))  # noqa
    def try_user(user: hug.directives.user):
        return user

    token = b'Basic ' + b64encode('{0}:{1}'.format('Tim', 'Custom password').encode('utf8'))
    assert hug.test.get(api, 'try_user', headers={'Authorization': token}).data == 'Tim'


def test_directives(hug_api):
    """Test to ensure cors directive works as expected"""
    assert hug.directives.cors('google.com') == 'google.com'

    @hug.get(api=hug_api)
    def cors_supported(cors: hug.directives.cors="*"):
        return True

    assert hug.test.get(hug_api, 'cors_supported').headers_dict['Access-Control-Allow-Origin'] == '*'



# pylint: disable=C0111, E0401
""" API Entry Point """

import hug


@hug.get("/", output=hug.output_format.html)
def base():
    return "<h1>hello world</h1>"


@hug.get("/add", examples="num=1")
def add(num: hug.types.number = 1):
    return {"res" : num + 1}



"""A simple example that illustrates returning UTF-8 encoded data within a JSON outputting hug endpoint"""
import hug


@hug.get()
def unicode_response():
    """An example endpoint that returns unicode data nested within the result object"""
    return {'data': ['Τη γλώσσα μου έδωσαν ελληνική']}



"""tests/test_output_format.py

Tests that the hug routing functionality works as expected

Copyright (C) 2016 Timothy Edmund Crosley

Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated
documentation files (the "Software"), to deal in the Software without restriction, including without limitation
the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and
to permit persons to whom the Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or
substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.

"""
import hug
from hug.routing import (CLIRouter, ExceptionRouter, HTTPRouter, InternalValidation, LocalRouter,
                         NotFoundRouter, Router, SinkRouter, StaticRouter, URLRouter)

api = hug.API(__name__)


class TestRouter(object):
    """A collection of tests to ensure the base Router object works as expected"""
    route = Router(transform='transform', output='output')

    def test_init(self):
        """Test to ensure the route instanciates as expected"""
        assert self.route.route['transform'] == 'transform'
        assert self.route.route['output'] == 'output'
        assert 'api' not in self.route.route

    def test_output(self):
        """Test to ensure modifying the output argument has the desired effect"""
        new_route = self.route.output('test data', transform='transformed')
        assert new_route != self.route
        assert new_route.route['output'] == 'test data'
        assert new_route.route['transform'] == 'transformed'

    def test_transform(self):
        """Test to ensure changing the transformation on the fly works as expected"""
        new_route = self.route.transform('transformed')
        assert new_route != self.route
        assert new_route.route['transform'] == 'transformed'

    def test_validate(self):
        """Test to ensure overriding the secondary validation method works as expected"""
        assert self.route.validate(str).route['validate'] == str

    def test_api(self):
        """Test to ensure changing the API associated with the route works as expected"""
        new_route = self.route.api('new')
        assert new_route != self.route
        assert new_route.route['api'] == 'new'

    def test_requires(self):
        """Test to ensure requirements can be added on the fly"""
        assert self.route.requires(('values', )).route['requires'] == ('values', )

    def test_map_params(self):
        """Test to ensure it is possible to set param mappings on the routing object"""
        assert self.route.map_params(id='user_id').route['map_params'] == {'id': 'user_id'}

    def test_where(self):
        """Test to ensure `where` can be used to replace all arguments on the fly"""
        new_route = self.route.where(transform='transformer', output='outputter')
        assert new_route != self.route
        assert new_route.route['output'] == 'outputter'
        assert new_route.route['transform'] == 'transformer'


class TestCLIRouter(TestRouter):
    """A collection of tests to ensure the CLIRouter object works as expected"""
    route = CLIRouter(name='cli', version=1, doc='Hi there!', transform='transform', output='output')

    def test_name(self):
        """Test to ensure the name can be replaced on the fly"""
        new_route = self.route.name('new name')
        assert new_route != self.route
        assert new_route.route['name'] == 'new name'
        assert new_route.route['transform'] == 'transform'
        assert new_route.route['output'] == 'output'

    def test_version(self):
        """Test to ensure the version can be replaced on the fly"""
        new_route = self.route.version(2)
        assert new_route != self.route
        assert new_route.route['version'] == 2
        assert new_route.route['transform'] == 'transform'
        assert new_route.route['output'] == 'output'

    def test_doc(self):
        """Test to ensure the documentation can be replaced on the fly"""
        new_route = self.route.doc('FAQ')
        assert new_route != self.route
        assert new_route.route['doc'] == 'FAQ'
        assert new_route.route['transform'] == 'transform'
        assert new_route.route['output'] == 'output'


class TestInternalValidation(TestRouter):
    """Collection of tests to ensure the base Router for routes that define internal validation work as expected"""
    route = InternalValidation(name='cli', doc='Hi there!', transform='transform', output='output')

    def test_raise_on_invalid(self):
        """Test to ensure it's possible to set a raise on invalid handler per route"""
        assert 'raise_on_invalid' not in self.route.route
        assert self.route.raise_on_invalid().route['raise_on_invalid']

    def test_on_invalid(self):
        """Test to ensure on_invalid handler can be changed on the fly"""
        assert self.route.on_invalid(str).route['on_invalid'] == str

    def test_output_invalid(self):
        """Test to ensure output_invalid handler can be changed on the fly"""
        assert self.route.output_invalid(hug.output_format.json).route['output_invalid'] == hug.output_format.json


class TestLocalRouter(TestInternalValidation):
    """A collection of tests to ensure the LocalRouter object works as expected"""
    route = LocalRouter(name='cli', doc='Hi there!', transform='transform', output='output')

    def test_validate(self):
        """Test to ensure changing wether a local route should validate or not works as expected"""
        assert 'skip_validation' not in self.route.route

        route = self.route.validate()
        assert 'skip_validation' not in route.route

        route = self.route.validate(False)
        assert 'skip_validation' in route.route

    def test_directives(self):
        """Test to ensure changing wether a local route should supply directives or not works as expected"""
        assert 'skip_directives' not in self.route.route

        route = self.route.directives()
        assert 'skip_directives' not in route.route

        route = self.route.directives(False)
        assert 'skip_directives' in route.route

    def test_version(self):
        """Test to ensure changing the version of a LocalRoute on the fly works"""
        assert 'version' not in self.route.route

        route = self.route.version(2)
        assert 'version' in route.route
        assert route.route['version'] == 2


class TestHTTPRouter(TestInternalValidation):
    """Collection of tests to ensure the base HTTPRouter object works as expected"""
    route = HTTPRouter(output='output', versions=(1, ), parse_body=False, transform='transform', requires=('love', ),
                       parameters=('one', ), defaults={'one': 'value'}, status=200)

    def test_versions(self):
        """Test to ensure the supported versions can be replaced on the fly"""
        assert self.route.versions(4).route['versions'] == (4, )

    def test_parse_body(self):
        """Test to ensure the parsing body flag be flipped on the fly"""
        assert self.route.parse_body().route['parse_body']
        assert 'parse_body' not in self.route.parse_body(False).route

    def test_requires(self):
        """Test to ensure requirements can be added on the fly"""
        assert self.route.requires(('values', )).route['requires'] == ('love', 'values')

    def test_doesnt_require(self):
        """Ensure requirements can be selectively removed on the fly"""
        assert self.route.doesnt_require('love').route.get('requires', ()) == ()
        assert self.route.doesnt_require('values').route['requires'] == ('love', )

        route = self.route.requires(('values', ))
        assert route.doesnt_require('love').route['requires'] == ('values', )
        assert route.doesnt_require('values').route['requires'] == ('love', )
        assert route.doesnt_require(('values', 'love')).route.get('requires', ()) == ()

    def test_parameters(self):
        """Test to ensure the parameters can be replaced on the fly"""
        assert self.route.parameters(('one', 'two')).route['parameters'] == ('one', 'two')

    def test_defaults(self):
        """Test to ensure the defaults can be replaced on the fly"""
        assert self.route.defaults({'one': 'three'}).route['defaults'] == {'one': 'three'}

    def test_status(self):
        """Test to ensure the default status can be changed on the fly"""
        assert self.route.set_status(500).route['status'] == 500


    def test_response_headers(self):
        """Test to ensure it's possible to switch out response headers for URL routes on the fly"""
        assert self.route.response_headers({'one': 'two'}).route['response_headers'] == {'one': 'two'}

    def test_add_response_headers(self):
        """Test to ensure it's possible to add headers on the fly"""
        route = self.route.response_headers({'one': 'two'})
        assert route.route['response_headers'] == {'one': 'two'}
        assert route.add_response_headers({'two': 'three'}).route['response_headers'] == {'one': 'two', 'two': 'three'}

    def test_cache(self):
        """Test to ensure it's easy to add a cache header on the fly"""
        assert self.route.cache().route['response_headers']['cache-control'] == 'public, max-age=31536000'

    def test_allow_origins(self):
        """Test to ensure it's easy to expose route to other resources"""
        test_headers = self.route.allow_origins(methods=('GET', 'POST'), credentials=True,
                                                headers="OPTIONS", max_age=10).route['response_headers']
        assert test_headers['Access-Control-Allow-Origin'] == '*'
        assert test_headers['Access-Control-Allow-Methods'] == 'GET, POST'
        assert test_headers['Access-Control-Allow-Credentials'] == 'true'
        assert test_headers['Access-Control-Allow-Headers'] == 'OPTIONS'
        assert test_headers['Access-Control-Max-Age'] == 10
        test_headers = self.route.allow_origins('google.com', methods=('GET', 'POST'), credentials=True,
                                                headers="OPTIONS", max_age=10).route['response_headers']
        assert 'Access-Control-Allow-Origin' not in test_headers
        assert test_headers['Access-Control-Allow-Methods'] == 'GET, POST'
        assert test_headers['Access-Control-Allow-Credentials'] == 'true'
        assert test_headers['Access-Control-Allow-Headers'] == 'OPTIONS'
        assert test_headers['Access-Control-Max-Age'] == 10


class TestStaticRouter(TestHTTPRouter):
    """Test to ensure that the static router sets up routes correctly"""
    route = StaticRouter("/here", requires=('love',), cache=True)
    route2 = StaticRouter(("/here", "/there"), api='api', cache={'no_store': True})

    def test_init(self):
        """Test to ensure the route instanciates as expected"""
        assert self.route.route['urls'] == ("/here", )
        assert self.route2.route['urls'] == ("/here", "/there")
        assert self.route2.route['api'] == 'api'


class TestSinkRouter(TestHTTPRouter):
    """Collection of tests to ensure that the SinkRouter works as expected"""
    route = SinkRouter(output='output', versions=(1, ), parse_body=False, transform='transform',
                       requires=('love', ), parameters=('one', ), defaults={'one': 'value'})


class TestNotFoundRouter(TestHTTPRouter):
    """Collection of tests to ensure the NotFoundRouter object works as expected"""
    route = NotFoundRouter(output='output', versions=(1, ), parse_body=False, transform='transform',
                           requires=('love', ), parameters=('one', ), defaults={'one': 'value'})


class TestExceptionRouter(TestHTTPRouter):
    """Collection of tests to ensure the ExceptionRouter object works as expected"""
    route = ExceptionRouter(Exception, output='output', versions=(1, ), parse_body=False, transform='transform',
                            requires=('love', ), parameters=('one', ), defaults={'one': 'value'})


class TestURLRouter(TestHTTPRouter):
    """Collection of tests to ensure the URLRouter object works as expected"""
    route = URLRouter('/here', transform='transform', output='output', requires=('love', ))

    def test_urls(self):
        """Test to ensure the url routes can be replaced on the fly"""
        assert self.route.urls('/there').route['urls'] == ('/there', )

    def test_accept(self):
        """Test to ensure the accept HTTP METHODs can be replaced on the fly"""
        assert self.route.accept('GET').route['accept'] == ('GET', )

    def test_get(self):
        """Test to ensure the HTTP METHOD can be set to just GET on the fly"""
        assert self.route.get().route['accept'] == ('GET', )
        assert self.route.get('/url').route['urls'] == ('/url', )

    def test_delete(self):
        """Test to ensure the HTTP METHOD can be set to just DELETE on the fly"""
        assert self.route.delete().route['accept'] == ('DELETE', )
        assert self.route.delete('/url').route['urls'] == ('/url', )

    def test_post(self):
        """Test to ensure the HTTP METHOD can be set to just POST on the fly"""
        assert self.route.post().route['accept'] == ('POST', )
        assert self.route.post('/url').route['urls'] == ('/url', )

    def test_put(self):
        """Test to ensure the HTTP METHOD can be set to just PUT on the fly"""
        assert self.route.put().route['accept'] == ('PUT', )
        assert self.route.put('/url').route['urls'] == ('/url', )

    def test_trace(self):
        """Test to ensure the HTTP METHOD can be set to just TRACE on the fly"""
        assert self.route.trace().route['accept'] == ('TRACE', )
        assert self.route.trace('/url').route['urls'] == ('/url', )

    def test_patch(self):
        """Test to ensure the HTTP METHOD can be set to just PATCH on the fly"""
        assert self.route.patch().route['accept'] == ('PATCH', )
        assert self.route.patch('/url').route['urls'] == ('/url', )

    def test_options(self):
        """Test to ensure the HTTP METHOD can be set to just OPTIONS on the fly"""
        assert self.route.options().route['accept'] == ('OPTIONS', )
        assert self.route.options('/url').route['urls'] == ('/url', )

    def test_head(self):
        """Test to ensure the HTTP METHOD can be set to just HEAD on the fly"""
        assert self.route.head().route['accept'] == ('HEAD', )
        assert self.route.head('/url').route['urls'] == ('/url', )

    def test_connect(self):
        """Test to ensure the HTTP METHOD can be set to just CONNECT on the fly"""
        assert self.route.connect().route['accept'] == ('CONNECT', )
        assert self.route.connect('/url').route['urls'] == ('/url', )

    def test_call(self):
        """Test to ensure the HTTP METHOD can be set to accept all on the fly"""
        assert self.route.call().route['accept'] == hug.HTTP_METHODS

    def test_http(self):
        """Test to ensure the HTTP METHOD can be set to accept all on the fly"""
        assert self.route.http().route['accept'] == hug.HTTP_METHODS

    def test_get_post(self):
        """Test to ensure the HTTP METHOD can be set to GET & POST in one call"""
        return self.route.get_post().route['accept'] == ('GET', 'POST')

    def test_put_post(self):
        """Test to ensure the HTTP METHOD can be set to PUT & POST in one call"""
        return self.route.put_post().route['accept'] == ('PUT', 'POST')

    def test_examples(self):
        """Test to ensure examples can be modified on the fly"""
        assert self.route.examples('none').route['examples'] == ('none', )

    def test_prefixes(self):
        """Test to ensure adding prefixes works as expected"""
        assert self.route.prefixes('/js/').route['prefixes'] == ('/js/', )

    def test_suffixes(self):
        """Test to ensure setting suffixes works as expected"""
        assert self.route.suffixes('.js', '.xml').route['suffixes'] == ('.js', '.xml')



import hug
from marshmallow import fields
from marshmallow.decorators import validates_schema
from marshmallow.schema import Schema
from marshmallow_sqlalchemy import ModelSchema

from demo.context import SqlalchemyContext
from demo.models import TestUser, TestModel


@hug.type(extend=hug.types.text, chain=True, accept_context=True)
def unique_username(value, context: SqlalchemyContext):
    if context.db.query(
        context.db.query(TestUser).filter(
            TestUser.username == value
        ).exists()
    ).scalar():
        raise ValueError('User with a username {0} already exists.'.format(value))
    return value


class CreateUserSchema(Schema):
    username = fields.String()
    password = fields.String()

    @validates_schema
    def check_unique_username(self, data):
        if self.context.db.query(
            self.context.db.query(TestUser).filter(
                    TestUser.username == data['username']
            ).exists()
        ).scalar():
            raise ValueError('User with a username {0} already exists.'.format(data['username']))


class DumpUserSchema(ModelSchema):

    @property
    def session(self):
        return self.context.db

    class Meta:
        model = TestModel
        fields = ('name',)


class DumpSchema(Schema):
    users = fields.Nested(DumpUserSchema, many=True)



import hug


def test_context_global_decorators(hug_api):
    custom_context = dict(context='global', factory=0, delete=0)

    @hug.context_factory(apply_globally=True)
    def create_context(*args, **kwargs):
        custom_context['factory'] += 1
        return custom_context

    @hug.delete_context(apply_globally=True)
    def delete_context(context, *args, **kwargs):
        assert context == custom_context
        custom_context['delete'] += 1

    @hug.get(api=hug_api)
    def made_up_hello():
        return 'hi'

    @hug.extend_api(api=hug_api, base_url='/api')
    def extend_with():
        import tests.module_fake_simple
        return (tests.module_fake_simple, )

    assert hug.test.get(hug_api, '/made_up_hello').data == 'hi'
    assert custom_context['factory'] == 1
    assert custom_context['delete'] == 1
    assert hug.test.get(hug_api, '/api/made_up_hello').data == 'hello'
    assert custom_context['factory'] == 2
    assert custom_context['delete'] == 2



import hug

from demo.context import SqlalchemyContext
from demo.models import TestUser


@hug.authentication.basic
def basic_authentication(username, password, context: SqlalchemyContext):
    return context.db.query(
        context.db.query(TestUser).filter(
            TestUser.username == username,
            TestUser.password == password
        ).exists()
    ).scalar()



"""Configuration for test environment"""
import sys

from .fixtures import *

collect_ignore = []

if sys.version_info < (3, 5):
    collect_ignore.append("test_async.py")

if sys.version_info < (3, 4):
    collect_ignore.append("test_coroutines.py")



"""This is an example of a hug "sink", these enable all request URLs that start with the one defined to be captured

To try this out, run this api with hug -f sink_example.py and hit any URL after localhost:8000/all/
(for example: localhost:8000/all/the/things/) and it will return the path sans the base URL. 
"""
import hug


@hug.sink('/all')
def my_sink(request):
    return request.path.replace('/all', '')



from sqlalchemy.engine import create_engine
from sqlalchemy.orm.scoping import scoped_session
from sqlalchemy.orm.session import sessionmaker, Session


engine = create_engine("sqlite:///:memory:")


session_factory = scoped_session(sessionmaker(bind=engine))


class SqlalchemyContext(object):

    def __init__(self):
        self._db = session_factory()

    @property
    def db(self) -> Session:
        return self._db
        # return self.session_factory()

    def cleanup(self, exception=None):
        if exception:
            self.db.rollback()
            return
        self.db.commit()



import hug

from demo.authentication import basic_authentication
from demo.directives import SqlalchemySession
from demo.models import TestUser, TestModel
from demo.validation import CreateUserSchema, DumpSchema, unique_username


@hug.post('/create_user2', requires=basic_authentication)
def create_user2(
    db: SqlalchemySession,
    data: CreateUserSchema()
):
    user = TestUser(
        **data
    )
    db.add(user)
    db.flush()
    return dict()


@hug.post('/create_user', requires=basic_authentication)
def create_user(
    db: SqlalchemySession,
    username: unique_username,
    password: hug.types.text
):
    user = TestUser(
        username=username,
        password=password
    )
    db.add(user)
    db.flush()
    return dict()


@hug.get('/test')
def test():
    return ''


@hug.get('/hello')
def make_simple_query(db: SqlalchemySession):
    for word in ["hello", "world", ":)"]:
        test_model = TestModel()
        test_model.name = word
        db.add(test_model)
        db.flush()
    return " ".join([obj.name for obj in db.query(TestModel).all()])


@hug.get('/hello2')
def transform_example(db: SqlalchemySession) -> DumpSchema():
    for word in ["hello", "world", ":)"]:
        test_model = TestModel()
        test_model.name = word
        db.add(test_model)
        db.flush()
    return dict(users=db.query(TestModel).all())


@hug.get('/protected', requires=basic_authentication)
def protected():
    return 'smile :)'



import time

from hug.decorators import auto_kwargs
from hug.introspect import generate_accepted_kwargs

DATA = {'request': None}


class Timer(object):

    def __init__(self, name):
        self.name = name

    def __enter__(self):
        self.start = time.time()

    def __exit__(self, *args):
        print("{0} took {1}".format(self.name, time.clock() - self.start))


def my_method(name, request=None):
    pass


def my_method_with_kwargs(name, request=None, **kwargs):
    pass


with Timer('generate_kwargs'):
    accept_kwargs = generate_accepted_kwargs(my_method, ('request', 'response', 'version'))

    for test in range(100000):
        my_method(test, **accept_kwargs(DATA))


with Timer('auto_kwargs'):
    wrapped_method = auto_kwargs(my_method)

    for test in range(100000):
        wrapped_method(test, **DATA)


with Timer('native_kwargs'):
    for test in range(100000):
        my_method_with_kwargs(test, **DATA)


with Timer('no_kwargs'):
    for test in range(100000):
        my_method(test, request=None)



"""Simple 1 endpoint Fake HUG API module usable for testing importation of modules"""
import hug


class FakeSimpleException(Exception):
    pass

@hug.get()
def made_up_hello():
    """for science!"""
    return 'hello'

@hug.get('/exception')
def made_up_exception():
    raise FakeSimpleException('test')



"""Provides an example of attaching an action on hug server startup"""
import hug

data = []


@hug.startup()
def add_data(api):
    """Adds initial data to the api on startup"""
    data.append("It's working")


@hug.startup()
def add_more_data(api):
    """Adds initial data to the api on startup"""
    data.append("Even subsequent calls")


@hug.cli()
@hug.get()
def test():
    """Returns all stored data"""
    return data



"""A simple file upload example.

To test, run this server with `hug -f file_upload_example.py`

Then run the following from ipython
(you may want to replace .wgetrc with some other small text file that you have,
and it's better to specify absolute path to it):

    import requests
    with open('.wgetrc', 'rb') as wgetrc_handle:
        response = requests.post('http://localhost:8000/upload', files={'.wgetrc': wgetrc_handle})
    print(response.headers)
    print(response.content)

This should both print in the terminal and return back the filename and filesize of the uploaded file.
"""

import hug

@hug.post('/upload')
def upload_file(body):
    """accepts file uploads"""
    # <body> is a simple dictionary of {filename: b'content'}
    print('body: ', body)
    return {'filename': list(body.keys()).pop(), 'filesize': len(list(body.values()).pop())}



"""An example of writing an API to scrape hacker news once, and then enabling usage everywhere"""
import hug
import requests


@hug.local()
@hug.cli()
@hug.get()
def top_post(section: hug.types.one_of(('news', 'newest', 'show'))='news'):
    """Returns the top post from the provided section"""
    content = requests.get('https://news.ycombinator.com/{0}'.format(section)).content
    text = content.decode('utf-8')
    return text.split('<tr class=\'athing\'>')[1].split("<a href")[1].split(">")[1].split("<")[0]



import hug

from demo import api
from demo.base import Base
from demo.context import SqlalchemyContext, engine
from demo.directives import SqlalchemySession
from demo.models import TestUser


@hug.context_factory()
def create_context(*args, **kwargs):
    return SqlalchemyContext()


@hug.delete_context()
def delete_context(context: SqlalchemyContext, exception=None, errors=None, lacks_requirement=None):
    context.cleanup(exception)


@hug.local(skip_directives=False)
def initialize(db: SqlalchemySession):
    admin = TestUser(username='admin', password='admin')
    db.add(admin)
    db.flush()


@hug.extend_api()
def apis():
    return [api]


Base.metadata.create_all(bind=engine)
initialize()



from pyramid.view import view_config
from pyramid.config import Configurator


@view_config(route_name='text', renderer='string')
def text(request):
    return 'Hello, World!'


config = Configurator()

config.add_route('text', '/text')

config.scan()
app = config.make_wsgi_app()



"""tests/test_output_format.py.

Tests the output format handlers included with Hug

Copyright (C) 2016 Timothy Edmund Crosley

Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated
documentation files (the "Software"), to deal in the Software without restriction, including without limitation
the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and
to permit persons to whom the Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or
substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.

"""
import numpy
import os
from collections import namedtuple
from datetime import datetime, timedelta
from decimal import Decimal
from io import BytesIO

import pytest

import hug

from .constants import BASE_DIRECTORY


def test_text():
    """Ensure that it's possible to output a Hug API method as text"""
    hug.output_format.text("Hello World!") == "Hello World!"
    hug.output_format.text(str(1)) == "1"


def test_html(hug_api):
    """Ensure that it's possible to output a Hug API method as HTML"""
    hug.output_format.html("<html>Hello World!</html>") == "<html>Hello World!</html>"
    hug.output_format.html(str(1)) == "1"
    with open(os.path.join(BASE_DIRECTORY, 'README.md'), 'rb') as html_file:
        assert hasattr(hug.output_format.html(html_file), 'read')

    class FakeHTMLWithRender():
        def render(self):
            return 'test'

    assert hug.output_format.html(FakeHTMLWithRender()) == b'test'

    @hug.get('/get/html', output=hug.output_format.html, api=hug_api)
    def get_html(**kwargs):
        """
        Returns command help document when no command is specified
        """
        with open(os.path.join(BASE_DIRECTORY, 'examples/document.html'), 'rb') as html_file:
            return html_file.read()

    assert '<html>' in hug.test.get(hug_api, '/get/html').data


def test_json():
    """Ensure that it's possible to output a Hug API method as JSON"""
    now = datetime.now()
    one_day = timedelta(days=1)
    test_data = {'text': 'text', 'datetime': now, 'bytes': b'bytes', 'delta': one_day}
    output = hug.output_format.json(test_data).decode('utf8')
    assert 'text' in output
    assert 'bytes' in output
    assert str(one_day.total_seconds()) in output
    assert now.isoformat() in output

    class NewObject(object):
        pass
    test_data['non_serializable'] = NewObject()
    with pytest.raises(TypeError):
        hug.output_format.json(test_data).decode('utf8')

    class NamedTupleObject(namedtuple('BaseTuple', ('name', 'value'))):
        pass
    data = NamedTupleObject('name', 'value')
    converted = hug.input_format.json(BytesIO(hug.output_format.json(data)))
    assert converted == {'name': 'name', 'value': 'value'}

    data = set((1, 2, 3, 3))
    assert hug.input_format.json(BytesIO(hug.output_format.json(data))) == [1, 2, 3]

    data = (number for number in range(1, 4))
    assert hug.input_format.json(BytesIO(hug.output_format.json(data))) == [1, 2, 3]

    data = [Decimal(1.5), Decimal("155.23"), Decimal("1234.25")]
    assert hug.input_format.json(BytesIO(hug.output_format.json(data))) == ["1.5", "155.23", "1234.25"]

    with open(os.path.join(BASE_DIRECTORY, 'README.md'), 'rb') as json_file:
        assert hasattr(hug.output_format.json(json_file), 'read')

    assert hug.input_format.json(BytesIO(hug.output_format.json(b'\x9c'))) == 'nA=='

    class MyCrazyObject(object):
        pass

    @hug.output_format.json_convert(MyCrazyObject)
    def convert(instance):
        return 'Like anyone could convert this'

    assert hug.input_format.json(BytesIO(hug.output_format.json(MyCrazyObject()))) == 'Like anyone could convert this'
    assert hug.input_format.json(BytesIO(hug.output_format.json({'data': ['Τη γλώσσα μου έδωσαν ελληνική']}))) == \
                                 {'data': ['Τη γλώσσα μου έδωσαν ελληνική']}


def test_pretty_json():
    """Ensure that it's possible to output a Hug API method as prettified and indented JSON"""
    test_data = {'text': 'text'}
    assert hug.output_format.pretty_json(test_data).decode('utf8') == ('{\n'
                                                                       '    "text": "text"\n'
                                                                       '}')


def test_json_camelcase():
    """Ensure that it's possible to output a Hug API method as camelCased JSON"""
    test_data = {'under_score': 'values_can', 'be_converted': [{'to_camelcase': 'value'}, 'wont_be_convert']}
    output = hug.output_format.json_camelcase(test_data).decode('utf8')
    assert 'underScore' in output
    assert 'values_can' in output
    assert 'beConverted' in output
    assert 'toCamelcase' in output
    assert 'value' in output
    assert 'wont_be_convert' in output


def test_image():
    """Ensure that it's possible to output images with hug"""
    logo_path = os.path.join(BASE_DIRECTORY, 'artwork', 'logo.png')
    assert hasattr(hug.output_format.png_image(logo_path, hug.Response()), 'read')
    with open(logo_path, 'rb') as image_file:
        assert hasattr(hug.output_format.png_image(image_file, hug.Response()), 'read')

    assert hug.output_format.png_image('Not Existent', hug.Response()) is None

    class FakeImageWithSave():
        def save(self, to, format):
            to.write(b'test')
    assert hasattr(hug.output_format.png_image(FakeImageWithSave(), hug.Response()), 'read')

    class FakeImageWithRender():
        def render(self):
            return 'test'
    assert hug.output_format.svg_xml_image(FakeImageWithRender(), hug.Response()) == 'test'

    class FakeImageWithSaveNoFormat():
        def save(self, to):
            to.write(b'test')
    assert hasattr(hug.output_format.png_image(FakeImageWithSaveNoFormat(), hug.Response()), 'read')


def test_file():
    """Ensure that it's possible to easily output files"""
    class FakeResponse(object):
        pass

    logo_path = os.path.join(BASE_DIRECTORY, 'artwork', 'logo.png')
    fake_response = FakeResponse()
    assert hasattr(hug.output_format.file(logo_path, fake_response), 'read')
    assert fake_response.content_type == 'image/png'
    with open(logo_path, 'rb') as image_file:
        hasattr(hug.output_format.file(image_file, fake_response), 'read')

    assert not hasattr(hug.output_format.file('NON EXISTENT FILE', fake_response), 'read')
    assert hug.output_format.file(None, fake_response) == ''


def test_video():
    """Ensure that it's possible to output videos with hug"""
    gif_path = os.path.join(BASE_DIRECTORY, 'artwork', 'example.gif')
    assert hasattr(hug.output_format.mp4_video(gif_path, hug.Response()), 'read')
    with open(gif_path, 'rb') as image_file:
        assert hasattr(hug.output_format.mp4_video(image_file, hug.Response()), 'read')

    assert hug.output_format.mp4_video('Not Existent', hug.Response()) is None

    class FakeVideoWithSave():
        def save(self, to, format):
            to.write(b'test')
    assert hasattr(hug.output_format.mp4_video(FakeVideoWithSave(), hug.Response()), 'read')

    class FakeVideoWithSave():
        def render(self):
            return 'test'
    assert hug.output_format.avi_video(FakeVideoWithSave(), hug.Response()) == 'test'


def test_on_valid():
    """Test to ensure formats that use on_valid content types gracefully handle error dictionaries"""
    error_dict = {'errors': {'so': 'many'}}
    expected = hug.output_format.json(error_dict)

    assert hug.output_format.mp4_video(error_dict, hug.Response()) == expected
    assert hug.output_format.png_image(error_dict, hug.Response()) == expected

    @hug.output_format.on_valid('image', hug.output_format.file)
    def my_output_format(data):
        raise ValueError('This should never be called')

    assert my_output_format(error_dict, hug.Response())


def test_on_content_type():
    """Ensure that it's possible to route the output type format by the requested content-type"""
    formatter = hug.output_format.on_content_type({'application/json': hug.output_format.json,
                                                   'text/plain': hug.output_format.text})

    class FakeRequest(object):
        content_type = 'application/json'

    request = FakeRequest()
    response = FakeRequest()
    converted = hug.input_format.json(formatter(BytesIO(hug.output_format.json({'name': 'name'})), request, response))
    assert converted == {'name': 'name'}

    request.content_type = 'text/plain'
    assert formatter('hi', request, response) == b'hi'

    with pytest.raises(hug.HTTPNotAcceptable):
        request.content_type = 'undefined; always'
        formatter('hi', request, response)


def test_accept():
    """Ensure that it's possible to route the output type format by the requests stated accept header"""
    formatter = hug.output_format.accept({'application/json': hug.output_format.json,
                                          'text/plain': hug.output_format.text})

    class FakeRequest(object):
        accept = 'application/json'

    request = FakeRequest()
    response = FakeRequest()
    converted = hug.input_format.json(formatter(BytesIO(hug.output_format.json({'name': 'name'})), request, response))
    assert converted == {'name': 'name'}

    request.accept = 'text/plain'
    assert formatter('hi', request, response) == b'hi'

    request.accept = 'application/json, text/plain; q=0.5'
    assert formatter('hi', request, response) == b'"hi"'

    request.accept = 'text/plain; q=0.5, application/json'
    assert formatter('hi', request, response) == b'"hi"'

    request.accept = 'application/json;q=0.4,text/plain; q=0.5'
    assert formatter('hi', request, response) == b'hi'

    request.accept = '*'
    assert formatter('hi', request, response) in [b'"hi"', b'hi']

    request.accept = 'undefined; always'
    with pytest.raises(hug.HTTPNotAcceptable):
        formatter('hi', request, response)


    formatter = hug.output_format.accept({'application/json': hug.output_format.json,
                                          'text/plain': hug.output_format.text}, hug.output_format.json)
    assert formatter('hi', request, response) == b'"hi"'


def test_suffix():
    """Ensure that it's possible to route the output type format by the suffix of the requested URL"""
    formatter = hug.output_format.suffix({'.js': hug.output_format.json, '.html': hug.output_format.text})

    class FakeRequest(object):
        path = 'endpoint.js'

    request = FakeRequest()
    response = FakeRequest()
    converted = hug.input_format.json(formatter(BytesIO(hug.output_format.json({'name': 'name'})), request, response))
    assert converted == {'name': 'name'}

    request.path = 'endpoint.html'
    assert formatter('hi', request, response) == b'hi'

    with pytest.raises(hug.HTTPNotAcceptable):
        request.path = 'undefined.always'
        formatter('hi', request, response)


def test_prefix():
    """Ensure that it's possible to route the output type format by the prefix of the requested URL"""
    formatter = hug.output_format.prefix({'js/': hug.output_format.json, 'html/': hug.output_format.text})

    class FakeRequest(object):
        path = 'js/endpoint'

    request = FakeRequest()
    response = FakeRequest()
    converted = hug.input_format.json(formatter(BytesIO(hug.output_format.json({'name': 'name'})), request, response))
    assert converted == {'name': 'name'}

    request.path = 'html/endpoint'
    assert formatter('hi', request, response) == b'hi'

    with pytest.raises(hug.HTTPNotAcceptable):
        request.path = 'undefined.always'
        formatter('hi', request, response)


def test_json_converter_numpy_types():
    """Ensure that numpy-specific data types (array, int, float) are properly supported in JSON output."""
    ex_np_array = numpy.array([1, 2, 3, 4, 5])
    ex_np_int = numpy.int_([5, 4, 3])
    ex_np_float = numpy.float(1.0)

    assert [1, 2, 3, 4, 5] == hug.output_format._json_converter(ex_np_array)
    assert [5, 4, 3] == hug.output_format._json_converter(ex_np_int)
    assert 1.0 == hug.output_format._json_converter(ex_np_float)


def test_output_format_with_no_docstring():
    """Ensure it is safe to use formatters with no docstring"""

    @hug.format.content_type('test/fmt')
    def test_fmt(data, request=None, response=None):
        return str(data).encode('utf8')

    hug.output_format.on_content_type({'test/fmt': test_fmt})



"""tests/test_api.py.

Tests to ensure the API object that stores the state of each individual hug endpoint works as expected

Copyright (C) 2016 Timothy Edmund Crosley

Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated
documentation files (the "Software"), to deal in the Software without restriction, including without limitation
the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and
to permit persons to whom the Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or
substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.

"""
import pytest

import hug

api = hug.API(__name__)


class TestAPI(object):
    """A collection of tests to ensure the hug API object interacts as expected"""

    def test_singleton(self):
        """Test to ensure there can only be one hug API per module"""
        assert hug.API(__name__) == api

    def test_context(self):
        """Test to ensure the hug singleton provides a global modifiable context"""
        assert not hasattr(hug.API(__name__), '_context')
        assert hug.API(__name__).context == {}
        assert hasattr(hug.API(__name__), '_context')

    def test_dynamic(self):
        """Test to ensure it's possible to dynamically create new modules to house APIs based on name alone"""
        new_api = hug.API('module_created_on_the_fly')
        assert new_api.module.__name__ == 'module_created_on_the_fly'
        import module_created_on_the_fly
        assert module_created_on_the_fly
        assert module_created_on_the_fly.__hug__ == new_api


def test_from_object():
    """Test to ensure it's possible to rechieve an API singleton from an arbitrary object"""
    assert hug.api.from_object(TestAPI) == api


def test_api_fixture(hug_api):
    """Ensure it's possible to dynamically insert a new hug API on demand"""
    assert isinstance(hug_api, hug.API)
    assert hug_api != api


def test_anonymous():
    """Ensure it's possible to create anonymous APIs"""
    assert hug.API() != hug.API() != api
    assert hug.API().module == None
    assert hug.API().name == ''
    assert hug.API(name='my_name').name == 'my_name'
    assert hug.API(doc='Custom documentation').doc == 'Custom documentation'


def test_api_routes(hug_api):
    """Ensure http API can return a quick mapping all urls to method"""
    hug_api.http.base_url = '/root'

    @hug.get(api=hug_api)
    def my_route():
        pass

    @hug.post(api=hug_api)
    def my_second_route():
        pass

    @hug.cli(api=hug_api)
    def my_cli_command():
        pass

    assert list(hug_api.http.urls()) == ['/root/my_route', '/root/my_second_route']
    assert list(hug_api.http.handlers()) == [my_route.interface.http, my_second_route.interface.http]
    assert list(hug_api.handlers()) == [my_route.interface.http, my_second_route.interface.http,
                                        my_cli_command.interface.cli]


def test_cli_interface_api_with_exit_codes(hug_api_error_exit_codes_enabled):
    api = hug_api_error_exit_codes_enabled

    @hug.object(api=api)
    class TrueOrFalse:
        @hug.object.cli
        def true(self):
            return True

        @hug.object.cli
        def false(self):
            return False

    api.cli(args=[None, 'true'])

    with pytest.raises(SystemExit):
        api.cli(args=[None, 'false'])


def test_cli_interface_api_without_exit_codes():
    @hug.object(api=api)
    class TrueOrFalse:
        @hug.object.cli
        def true(self):
            return True

        @hug.object.cli
        def false(self):
            return False

    api.cli(args=[None, 'true'])
    api.cli(args=[None, 'false'])



import hug
from sqlalchemy.orm.session import Session

from demo.context import SqlalchemyContext


@hug.directive()
class SqlalchemySession(Session):

    def __new__(cls, *args, context: SqlalchemyContext=None, **kwargs):
        return context.db



"""tests/test_store.py.

Tests to ensure that the native stores work correctly.

Copyright (C) 2016 Timothy Edmund Crosley

Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated
documentation files (the "Software"), to deal in the Software without restriction, including without limitation
the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and
to permit persons to whom the Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or
substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.

"""
import pytest

from hug.exceptions import StoreKeyNotFound
from hug.store import InMemoryStore

stores_to_test = [
    InMemoryStore()
]


@pytest.mark.parametrize('store', stores_to_test)
def test_stores_generically(store):
    key = 'test-key'
    data = {
        'user': 'foo',
        'authenticated': False
    }

    # Key should not exist
    assert not store.exists(key)

    # Set key with custom data, verify the key exists and expect correct data to be returned
    store.set(key, data)
    assert store.exists(key)
    assert store.get(key) == data

    # Expect exception if unknown session key was requested
    with pytest.raises(StoreKeyNotFound):
        store.get('unknown')

    # Delete key
    store.delete(key)
    assert not store.exists(key)



"""tests/test_use.py.

Tests to ensure hugs service consuming classes, that are the backbone of the seamless micro-service concept,
work as intended

Copyright (C) 2016 Timothy Edmund Crosley

Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated
documentation files (the "Software"), to deal in the Software without restriction, including without limitation
the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and
to permit persons to whom the Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or
substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.

"""
import socket
import struct

import pytest
import requests

import hug
from hug import use


class TestService(object):
    """Test to ensure the base Service object works as a base Abstract service runner"""
    service = use.Service(version=1, timeout=100, raise_on=(500, ))

    def test_init(self):
        """Test to ensure base service instantiation populates expected attributes"""
        assert self.service.version == 1
        assert self.service.raise_on == (500, )
        assert self.service.timeout == 100

    def test_request(self):
        """Test to ensure the abstract service request method raises NotImplementedError to show its abstract nature"""
        with pytest.raises(NotImplementedError):
            self.service.request('POST', 'endpoint')

    def test_get(self):
        """Test to ensure the abstract service get method raises NotImplementedError to show its abstract nature"""
        with pytest.raises(NotImplementedError):
            self.service.get('endpoint')

    def test_post(self):
        """Test to ensure the abstract service post method raises NotImplementedError to show its abstract nature"""
        with pytest.raises(NotImplementedError):
            self.service.post('endpoint')

    def test_delete(self):
        """Test to ensure the abstract service delete method raises NotImplementedError to show its abstract nature"""
        with pytest.raises(NotImplementedError):
            self.service.delete('endpoint')

    def test_put(self):
        """Test to ensure the abstract service put method raises NotImplementedError to show its abstract nature"""
        with pytest.raises(NotImplementedError):
            self.service.put('endpoint')

    def test_trace(self):
        """Test to ensure the abstract service trace method raises NotImplementedError to show its abstract nature"""
        with pytest.raises(NotImplementedError):
            self.service.trace('endpoint')

    def test_patch(self):
        """Test to ensure the abstract service patch method raises NotImplementedError to show its abstract nature"""
        with pytest.raises(NotImplementedError):
            self.service.patch('endpoint')

    def test_options(self):
        """Test to ensure the abstract service options method raises NotImplementedError to show its abstract nature"""
        with pytest.raises(NotImplementedError):
            self.service.options('endpoint')

    def test_head(self):
        """Test to ensure the abstract service head method raises NotImplementedError to show its abstract nature"""
        with pytest.raises(NotImplementedError):
            self.service.head('endpoint')

    def test_connect(self):
        """Test to ensure the abstract service connect method raises NotImplementedError to show its abstract nature"""
        with pytest.raises(NotImplementedError):
            self.service.connect('endpoint')


class TestHTTP(object):
    """Test to ensure the HTTP Service object enables pulling data from external HTTP services"""
    service = use.HTTP('http://www.google.com/', raise_on=(404, 400))
    url_service = use.HTTP('http://www.google.com/', raise_on=(404, 400), json_transport=False)

    def test_init(self):
        """Test to ensure HTTP service instantiation populates expected attributes"""
        assert self.service.endpoint == 'http://www.google.com/'
        assert self.service.raise_on == (404, 400)

    @pytest.mark.extnetwork
    def test_request(self):
        """Test so ensure the HTTP service can successfully be used to pull data from an external service"""
        response = self.url_service.request('GET', 'search', query='api')
        assert response
        assert response.data

        with pytest.raises(requests.HTTPError):
            response = self.service.request('GET', 'search', query='api')

        with pytest.raises(requests.HTTPError):
            self.url_service.request('GET', 'not_found', query='api')


class TestLocal(object):
    """Test to ensure the Local Service object enables pulling data from internal hug APIs with minimal overhead"""
    service = use.Local(__name__)

    def test_init(self):
        """Test to ensure the Local service instantiation populates the expected attributes"""
        assert isinstance(self.service.api, hug.API)

    def test_request(self):
        """Test to ensure requesting data from a local service works as expected"""
        assert self.service.get('hello_world').data == 'Hi!'
        assert self.service.get('not_there').status_code == 404
        assert self.service.get('validation_error').status_code == 400

        self.service.raise_on = (404, 500)
        with pytest.raises(requests.HTTPError):
            assert self.service.get('not_there')

        with pytest.raises(requests.HTTPError):
            assert self.service.get('exception')


class TestSocket(object):
    """Test to ensure the Socket Service object enables sending/receiving data from arbitrary server/port sockets"""
    on_unix = getattr(socket, 'AF_UNIX', False)
    tcp_service = use.Socket(connect_to=('www.google.com', 80), proto='tcp', timeout=60)
    udp_service = use.Socket(connect_to=('8.8.8.8', 53), proto='udp', timeout=60)

    def test_init(self):
        """Test to ensure the Socket service instantiation populates the expected attributes"""
        assert isinstance(self.tcp_service, use.Service)

    def test_protocols(self):
        """Test to ensure all supported protocols are present"""
        protocols = sorted(['tcp', 'udp', 'unix_stream', 'unix_dgram'])
        if self.on_unix:
            assert sorted(self.tcp_service.protocols) == protocols
        else:
            protocols.remove('unix_stream')
            protocols.remove('unix_dgram')
            assert sorted(self.tcp_service.protocols) == protocols

    def test_streams(self):
        if self.on_unix:
            assert set(self.tcp_service.streams) == set(('tcp', 'unix_stream', ))
        else:
            assert set(self.tcp_service.streams) == set(('tcp', ))

    def test_datagrams(self):
        if self.on_unix:
            assert set(self.tcp_service.datagrams) == set(('udp', 'unix_dgram', ))
        else:
            assert set(self.tcp_service.datagrams) == set(('udp', ))

    def test_inet(self):
        assert set(self.tcp_service.inet) == set(('tcp', 'udp', ))

    def test_unix(self):
        if self.on_unix:
            assert set(self.tcp_service.unix) == set(('unix_stream', 'unix_dgram', ))
        else:
            assert set(self.tcp_service.unix) == set()

    def test_connection(self):
        assert self.tcp_service.connection.connect_to == ('www.google.com', 80)
        assert self.tcp_service.connection.proto == 'tcp'
        assert set(self.tcp_service.connection.sockopts) == set()

    def test_settimeout(self):
        self.tcp_service.settimeout(60)
        assert self.tcp_service.timeout == 60

    def test_connection_sockopts_unit(self):
        self.tcp_service.connection.sockopts.clear()
        self.tcp_service.setsockopt(socket.SOL_SOCKET, socket.SO_KEEPALIVE, 1)
        assert self.tcp_service.connection.sockopts == {(socket.SOL_SOCKET, socket.SO_KEEPALIVE, 1)}

    def test_connection_sockopts_batch(self):
        self.tcp_service.setsockopt(((socket.SOL_SOCKET, socket.SO_KEEPALIVE, 1),
                                 (socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)))
        assert self.tcp_service.connection.sockopts == {(socket.SOL_SOCKET, socket.SO_KEEPALIVE, 1),
                                                        (socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)}

    @pytest.mark.extnetwork
    def test_datagram_request(self):
        """Test to ensure requesting data from a socket service works as expected"""
        packet = struct.pack("!HHHHHH", 0x0001, 0x0100, 1, 0, 0, 0)
        for name in ('www', 'google', 'com'):
            header = b"!b"
            header += bytes(str(len(name)), "utf-8") + b"s"
            query = struct.pack(header, len(name), name.encode('utf-8'))
            packet = packet + query

        dns_query = packet + struct.pack("!bHH", 0, 1, 1)
        assert len(self.udp_service.request(dns_query.decode("utf-8"), buffer_size=4096).data.read()) > 0


@hug.get()
def hello_world():
    return 'Hi!'


@hug.get()
def exception(response):
    response.status = hug.HTTP_500


@hug.get()
def validation_error(data):
    return data



"""A simple example of a hug API call with versioning"""
import hug


@hug.get('/echo', versions=1)
def echo(text):
    return text


@hug.get('/echo', versions=range(2, 5))  # noqa
def echo(text):
    return 'Echo: {text}'.format(**locals())


@hug.get('/unversioned')
def hello():
    return 'Hello world!'


@hug.get('/echo', versions='6')
def echo(text):
    return 'Version 6'



import os

import hug


DIRECTORY = os.path.dirname(os.path.realpath(__file__))


@hug.get('/get/document', output=hug.output_format.html)
def nagiosCommandHelp(**kwargs):
    """
    Returns command help document when no command is specified
    """
    with open(os.path.join(DIRECTORY, 'document.html')) as document:
        return document.read()



"""A basic (single function) API written using Hug"""
import hug


@hug.get('/happy_birthday', examples="name=HUG&age=1")
def happy_birthday(name, age: hug.types.number):
    """Says happy birthday to a user"""
    return "Happy {age} Birthday {name}!".format(**locals())

@hug.get('/greet/{event}')
def greet(event: str):
    """Greets appropriately (from http://blog.ketchum.com/how-to-write-10-common-holiday-greetings/)  """
    greetings = "Happy"
    if event == "Christmas":
        greetings = "Merry"
    if event == "Kwanzaa":
        greetings = "Joyous"
    if event == "wishes":
        greetings = "Warm"

    return "{greetings} {event}!".format(**locals())



"""tests/test_input_format.py.

Tests that hugs built in introspection helper functions work as expected

Copyright (C) 2016 Timothy Edmund Crosley

Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated
documentation files (the "Software"), to deal in the Software without restriction, including without limitation
the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and
to permit persons to whom the Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or
substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.

"""
import hug


def function_with_kwargs(argument1, **kwargs):
    pass


def function_with_args(argument1, *args):
    pass


def function_with_neither(argument1, argument2):
    pass


def function_with_both(argument1, argument2, argument3, *args, **kwargs):
    pass


def function_with_nothing():
    pass


class Object(object):

    def my_method(self):
        pass


def test_is_method():
    """Test to ensure hugs introspection can correctly identify the difference between a function and method"""
    assert not hug.introspect.is_method(function_with_kwargs)
    assert hug.introspect.is_method(Object().my_method)


def test_arguments():
    """Test to ensure hug introspection can correctly pull out arguments from a function definition"""
    def function(argument1, argument2):
        pass

    assert tuple(hug.introspect.arguments(function_with_kwargs)) == ('argument1', )
    assert tuple(hug.introspect.arguments(function_with_args)) == ('argument1', )
    assert tuple(hug.introspect.arguments(function_with_neither)) == ('argument1', 'argument2')
    assert tuple(hug.introspect.arguments(function_with_both)) == ('argument1', 'argument2', 'argument3')


def test_takes_kwargs():
    """Test to ensure hug introspection can correctly identify when a function takes kwargs"""
    assert hug.introspect.takes_kwargs(function_with_kwargs)
    assert not hug.introspect.takes_kwargs(function_with_args)
    assert not hug.introspect.takes_kwargs(function_with_neither)
    assert hug.introspect.takes_kwargs(function_with_both)


def test_takes_args():
    """Test to ensure hug introspection can correctly identify when a function takes args"""
    assert not hug.introspect.takes_args(function_with_kwargs)
    assert hug.introspect.takes_args(function_with_args)
    assert not hug.introspect.takes_args(function_with_neither)
    assert hug.introspect.takes_args(function_with_both)


def test_takes_arguments():
    """Test to ensure hug introspection can correctly identify which arguments supplied a function will take"""
    assert hug.introspect.takes_arguments(function_with_kwargs, 'argument1', 'argument3') == set(('argument1', ))
    assert hug.introspect.takes_arguments(function_with_args, 'bacon') == set()
    assert hug.introspect.takes_arguments(function_with_neither,
                                          'argument1', 'argument2') == set(('argument1', 'argument2'))
    assert hug.introspect.takes_arguments(function_with_both, 'argument3', 'bacon') == set(('argument3', ))


def test_takes_all_arguments():
    """Test to ensure hug introspection can correctly identify if a function takes all specified arguments"""
    assert not hug.introspect.takes_all_arguments(function_with_kwargs, 'argument1', 'argument2', 'argument3')
    assert not hug.introspect.takes_all_arguments(function_with_args, 'argument1', 'argument2', 'argument3')
    assert not hug.introspect.takes_all_arguments(function_with_neither, 'argument1', 'argument2', 'argument3')
    assert hug.introspect.takes_all_arguments(function_with_both, 'argument1', 'argument2', 'argument3')


def test_generate_accepted_kwargs():
    """Test to ensure hug introspection can correctly dynamically filter out kwargs for only those accepted"""
    source_dictionary = {'argument1': 1, 'argument2': 2, 'hey': 'there', 'hi': 'hello'}

    kwargs = hug.introspect.generate_accepted_kwargs(function_with_kwargs, 'bacon', 'argument1')(source_dictionary)
    assert kwargs == source_dictionary

    kwargs = hug.introspect.generate_accepted_kwargs(function_with_args, 'bacon', 'argument1')(source_dictionary)
    assert kwargs == {'argument1': 1}

    kwargs = hug.introspect.generate_accepted_kwargs(function_with_neither, 'argument1', 'argument2')(source_dictionary)
    assert kwargs == {'argument1': 1, 'argument2': 2}

    kwargs = hug.introspect.generate_accepted_kwargs(function_with_both, 'argument1', 'argument2')(source_dictionary)
    assert kwargs == source_dictionary

    kwargs = hug.introspect.generate_accepted_kwargs(function_with_nothing)(source_dictionary)
    assert kwargs == {}



from pymongo import MongoClient
import hug


client = MongoClient('db', 27017)
db = client['our-database']
collection = db['our-items']


@hug.get('/', output=hug.output_format.pretty_json)
def show():
    """Returns a list of items currently in the database"""
    items = list(collection.find())
    # JSON conversion chokes on the _id objects, so we convert
    # them to strings here
    for i in items:
        i['_id'] = str(i['_id'])
    return items


@hug.post('/new', status_code=hug.falcon.HTTP_201)
def new(name: hug.types.text, description: hug.types.text):
    """Inserts the given object as a new item in the database.

    Returns the ID of the newly created item.
    """
    item_doc = {'name': name, 'description': description}
    collection.insert_one(item_doc)
    return str(item_doc['_id'])




import envelopes
import hug


@hug.directive()
class SMTP(object):

    def __init__(self, *args, **kwargs):
        self.smtp = envelopes.SMTP(host='127.0.0.1')
        self.envelopes_to_send = list()

    def send_envelope(self, envelope):
        self.envelopes_to_send.append(envelope)

    def cleanup(self, exception=None):
        if exception:
            return
        for envelope in self.envelopes_to_send:
            self.smtp.send(envelope)


@hug.get('/hello')
def send_hello_email(smtp: SMTP):
    envelope = envelopes.Envelope(
        from_addr=(u'me@example.com', u'From me'),
        to_addr=(u'world@example.com', u'To World'),
        subject=u'Hello',
        text_body=u"World!"
    )
    smtp.send_envelope(envelope)



import sys

from marshmallow.decorators import post_dump

import hug
from marshmallow import fields, Schema
import pytest


module = sys.modules[__name__]


class RequirementFailed(object):

    def __str__(self):
        return "requirement failed"


class CustomException(Exception):
    pass


class TestContextFactoryLocal(object):

    def test_lack_requirement(self):
        self.custom_context = dict(test='context')

        @hug.context_factory()
        def return_context(**kwargs):
            return self.custom_context

        @hug.delete_context()
        def delete_context(context, exception=None, errors=None, lacks_requirement=None):
            assert context == self.custom_context
            assert not exception
            assert not errors
            assert lacks_requirement
            assert isinstance(lacks_requirement, RequirementFailed)
            self.custom_context['launched_delete_context'] = True

        def test_local_requirement(**kwargs):
            assert 'context' in kwargs
            assert kwargs['context'] == self.custom_context
            self.custom_context['launched_requirement'] = True
            return RequirementFailed()

        @hug.local(requires=test_local_requirement)
        def requirement_local_function():
            self.custom_context['launched_local_function'] = True

        requirement_local_function()
        assert 'launched_local_function' not in self.custom_context
        assert 'launched_requirement' in self.custom_context
        assert 'launched_delete_context' in self.custom_context

    def test_directive(self):
        custom_context = dict(test='context')

        @hug.context_factory()
        def return_context(**kwargs):
            return custom_context

        @hug.delete_context()
        def delete_context(context, **kwargs):
            pass

        @hug.directive()
        def custom_directive(**kwargs):
            assert 'context' in kwargs
            assert kwargs['context'] == custom_context
            return 'custom'

        @hug.local()
        def directive_local_function(custom: custom_directive):
            assert custom == 'custom'

        directive_local_function()

    def test_validation(self):
        custom_context = dict(test='context', not_valid_number=43)

        @hug.context_factory()
        def return_context(**kwargs):
            return custom_context

        @hug.delete_context()
        def delete_context(context, exception=None, errors=None, lacks_requirement=None):
            assert context == custom_context
            assert not exception
            assert errors
            assert not lacks_requirement
            custom_context['launched_delete_context'] = True

        def test_requirement(**kwargs):
            assert 'context' in kwargs
            assert kwargs['context'] == custom_context
            custom_context['launched_requirement'] = True
            return RequirementFailed()

        @hug.type(extend=hug.types.number, accept_context=True)
        def custom_number_test(value, context):
            assert context == custom_context
            if value == context['not_valid_number']:
                raise ValueError('not valid number')
            return value

        @hug.local()
        def validation_local_function(value: custom_number_test):
            custom_context['launched_local_function'] = value

        validation_local_function(43)
        assert not 'launched_local_function' in custom_context
        assert 'launched_delete_context' in custom_context

    def test_transform(self):
        custom_context = dict(test='context', test_number=43)

        @hug.context_factory()
        def return_context(**kwargs):
            return custom_context

        @hug.delete_context()
        def delete_context(context, exception=None, errors=None, lacks_requirement=None):
            assert context == custom_context
            assert not exception
            assert not errors
            assert not lacks_requirement
            custom_context['launched_delete_context'] = True

        class UserSchema(Schema):
            name = fields.Str()

            @post_dump()
            def check_context(self, data):
                assert self.context['test'] == 'context'
                self.context['test_number'] += 1

        @hug.local()
        def validation_local_function() -> UserSchema():
            return {'name': 'test'}

        validation_local_function()
        assert 'test_number' in custom_context and custom_context['test_number'] == 44
        assert 'launched_delete_context' in custom_context

    def test_exception(self):
        custom_context = dict(test='context')

        @hug.context_factory()
        def return_context(**kwargs):
            return custom_context

        @hug.delete_context()
        def delete_context(context, exception=None, errors=None, lacks_requirement=None):
            assert context == custom_context
            assert exception
            assert isinstance(exception, CustomException)
            assert not errors
            assert not lacks_requirement
            custom_context['launched_delete_context'] = True

        @hug.local()
        def exception_local_function():
            custom_context['launched_local_function'] = True
            raise CustomException()

        with pytest.raises(CustomException):
            exception_local_function()

        assert 'launched_local_function' in custom_context
        assert 'launched_delete_context' in custom_context

    def test_success(self):
        custom_context = dict(test='context')

        @hug.context_factory()
        def return_context(**kwargs):
            return custom_context

        @hug.delete_context()
        def delete_context(context, exception=None, errors=None, lacks_requirement=None):
            assert context == custom_context
            assert not exception
            assert not errors
            assert not lacks_requirement
            custom_context['launched_delete_context'] = True

        @hug.local()
        def success_local_function():
            custom_context['launched_local_function'] = True

        success_local_function()

        assert 'launched_local_function' in custom_context
        assert 'launched_delete_context' in custom_context


class TestContextFactoryCLI(object):

    def test_lack_requirement(self):
        custom_context = dict(test='context')

        @hug.context_factory()
        def return_context(**kwargs):
            return custom_context

        @hug.delete_context()
        def delete_context(context, exception=None, errors=None, lacks_requirement=None):
            assert context == custom_context
            assert not exception
            assert not errors
            assert lacks_requirement
            assert isinstance(lacks_requirement, RequirementFailed)
            custom_context['launched_delete_context'] = True

        def test_requirement(**kwargs):
            assert 'context' in kwargs
            assert kwargs['context'] == custom_context
            custom_context['launched_requirement'] = True
            return RequirementFailed()

        @hug.cli(requires=test_requirement)
        def requirement_local_function():
            custom_context['launched_local_function'] = True

        hug.test.cli(requirement_local_function)
        assert 'launched_local_function' not in custom_context
        assert 'launched_requirement' in custom_context
        assert 'launched_delete_context' in custom_context

    def test_directive(self):
        custom_context = dict(test='context')

        @hug.context_factory()
        def return_context(**kwargs):
            return custom_context

        @hug.delete_context()
        def delete_context(context, **kwargs):
            pass

        @hug.directive()
        def custom_directive(**kwargs):
            assert 'context' in kwargs
            assert kwargs['context'] == custom_context
            return 'custom'

        @hug.cli()
        def directive_local_function(custom: custom_directive):
            assert custom == 'custom'

        hug.test.cli(directive_local_function)

    def test_validation(self):
        custom_context = dict(test='context', not_valid_number=43)

        @hug.context_factory()
        def return_context(**kwargs):
            return custom_context

        @hug.delete_context()
        def delete_context(context, exception=None, errors=None, lacks_requirement=None):
            assert not exception
            assert context == custom_context
            assert errors
            assert not lacks_requirement
            custom_context['launched_delete_context'] = True

        def test_requirement(**kwargs):
            assert 'context' in kwargs
            assert kwargs['context'] == custom_context
            custom_context['launched_requirement'] = True
            return RequirementFailed()

        @hug.type(extend=hug.types.number, accept_context=True)
        def new_custom_number_test(value, context):
            assert context == custom_context
            if value == context['not_valid_number']:
                raise ValueError('not valid number')
            return value

        @hug.cli()
        def validation_local_function(value: hug.types.number):
            custom_context['launched_local_function'] = value
            return 0

        with pytest.raises(SystemExit):
            hug.test.cli(validation_local_function, 'xxx')
        assert 'launched_local_function' not in custom_context
        assert 'launched_delete_context' in custom_context

    def test_transform(self):
        custom_context = dict(test='context', test_number=43)

        @hug.context_factory()
        def return_context(**kwargs):
            return custom_context

        @hug.delete_context()
        def delete_context(context, exception=None, errors=None, lacks_requirement=None):
            assert not exception
            assert context == custom_context
            assert not errors
            assert not lacks_requirement
            custom_context['launched_delete_context'] = True

        class UserSchema(Schema):
            name = fields.Str()

            @post_dump()
            def check_context(self, data):
                assert self.context['test'] == 'context'
                self.context['test_number'] += 1

        @hug.cli()
        def transform_cli_function() -> UserSchema():
            custom_context['launched_cli_function'] = True
            return {'name': 'test'}

        hug.test.cli(transform_cli_function)
        assert 'launched_cli_function' in custom_context
        assert 'launched_delete_context' in custom_context
        assert 'test_number' in custom_context
        assert custom_context['test_number'] == 44

    def test_exception(self):
        custom_context = dict(test='context')

        @hug.context_factory()
        def return_context(**kwargs):
            return custom_context

        @hug.delete_context()
        def delete_context(context, exception=None, errors=None, lacks_requirement=None):
            assert context == custom_context
            assert exception
            assert isinstance(exception, CustomException)
            assert not errors
            assert not lacks_requirement
            custom_context['launched_delete_context'] = True

        @hug.cli()
        def exception_local_function():
            custom_context['launched_local_function'] = True
            raise CustomException()

        hug.test.cli(exception_local_function)

        assert 'launched_local_function' in custom_context
        assert 'launched_delete_context' in custom_context

    def test_success(self):
        custom_context = dict(test='context')

        @hug.context_factory()
        def return_context(**kwargs):
            return custom_context

        @hug.delete_context()
        def delete_context(context, exception=None, errors=None, lacks_requirement=None):
            assert context == custom_context
            assert not exception
            assert not errors
            assert not lacks_requirement
            custom_context['launched_delete_context'] = True

        @hug.cli()
        def success_local_function():
            custom_context['launched_local_function'] = True

        hug.test.cli(success_local_function)

        assert 'launched_local_function' in custom_context
        assert 'launched_delete_context' in custom_context


class TestContextFactoryHTTP(object):

    def test_lack_requirement(self):
        custom_context = dict(test='context')

        @hug.context_factory()
        def return_context(**kwargs):
            return custom_context

        @hug.delete_context()
        def delete_context(context, exception=None, errors=None, lacks_requirement=None):
            assert context == custom_context
            assert not exception
            assert not errors
            assert lacks_requirement
            custom_context['launched_delete_context'] = True

        def test_requirement(**kwargs):
            assert 'context' in kwargs
            assert kwargs['context'] == custom_context
            custom_context['launched_requirement'] = True
            return 'requirement_failed'

        @hug.get('/requirement_function', requires=test_requirement)
        def requirement_http_function():
            custom_context['launched_local_function'] = True

        hug.test.get(module, '/requirement_function')
        assert 'launched_local_function' not in custom_context
        assert 'launched_requirement' in custom_context
        assert 'launched_delete_context' in custom_context

    def test_directive(self):
        custom_context = dict(test='context')

        @hug.context_factory()
        def return_context(**kwargs):
            return custom_context

        @hug.delete_context()
        def delete_context(context, **kwargs):
            pass

        @hug.directive()
        def custom_directive(**kwargs):
            assert 'context' in kwargs
            assert kwargs['context'] == custom_context
            return 'custom'

        @hug.get('/directive_function')
        def directive_http_function(custom: custom_directive):
            assert custom == 'custom'

        hug.test.get(module, '/directive_function')

    def test_validation(self):
        custom_context = dict(test='context', not_valid_number=43)

        @hug.context_factory()
        def return_context(**kwargs):
            return custom_context

        @hug.delete_context()
        def delete_context(context, exception=None, errors=None, lacks_requirement=None):
            assert context == custom_context
            assert not exception
            assert errors
            assert not lacks_requirement
            custom_context['launched_delete_context'] = True

        def test_requirement(**kwargs):
            assert 'context' in kwargs
            assert kwargs['context'] == custom_context
            custom_context['launched_requirement'] = True
            return RequirementFailed()

        @hug.type(extend=hug.types.number, accept_context=True)
        def custom_number_test(value, context):
            assert context == custom_context
            if value == context['not_valid_number']:
                raise ValueError('not valid number')
            return value

        @hug.get('/validation_function')
        def validation_http_function(value: custom_number_test):
            custom_context['launched_local_function'] = value

        hug.test.get(module, '/validation_function', 43)
        assert 'launched_local_function ' not in custom_context
        assert 'launched_delete_context' in custom_context

    def test_transform(self):
        custom_context = dict(test='context', test_number=43)

        @hug.context_factory()
        def return_context(**kwargs):
            return custom_context

        @hug.delete_context()
        def delete_context(context, exception=None, errors=None, lacks_requirement=None):
            assert context == custom_context
            assert not exception
            assert not errors
            assert not lacks_requirement
            custom_context['launched_delete_context'] = True

        class UserSchema(Schema):
            name = fields.Str()

            @post_dump()
            def check_context(self, data):
                assert self.context['test'] == 'context'
                self.context['test_number'] += 1

        @hug.get('/validation_function')
        def validation_http_function() -> UserSchema():
            custom_context['launched_local_function'] = True

        hug.test.get(module, '/validation_function', 43)
        assert 'launched_local_function' in custom_context
        assert 'launched_delete_context' in custom_context
        assert 'test_number' in custom_context
        assert custom_context['test_number'] == 44

    def test_exception(self):
        custom_context = dict(test='context')

        @hug.context_factory()
        def return_context(**kwargs):
            return custom_context

        @hug.delete_context()
        def delete_context(context, exception=None, errors=None, lacks_requirement=None):
            assert context == custom_context
            assert exception
            assert isinstance(exception, CustomException)
            assert not errors
            assert not lacks_requirement
            custom_context['launched_delete_context'] = True

        @hug.get('/exception_function')
        def exception_http_function():
            custom_context['launched_local_function'] = True
            raise CustomException()

        with pytest.raises(CustomException):
            hug.test.get(module, '/exception_function')

        assert 'launched_local_function' in custom_context
        assert 'launched_delete_context' in custom_context

    def test_success(self):
        custom_context = dict(test='context')

        @hug.context_factory()
        def return_context(**kwargs):
            return custom_context

        @hug.delete_context()
        def delete_context(context, exception=None, errors=None, lacks_requirement=None):
            assert context == custom_context
            assert not exception
            assert not errors
            assert not lacks_requirement
            custom_context['launched_delete_context'] = True

        @hug.get('/success_function')
        def success_http_function():
            custom_context['launched_local_function'] = True

        hug.test.get(module, '/success_function')

        assert 'launched_local_function' in custom_context
        assert 'launched_delete_context' in custom_context



"""Serves a directory from the filesystem using Hug.

try /static/a/hi.txt  /static/a/hi.html  /static/a/hello.html
"""
import tempfile
import os

import hug

tmp_dir_object = None

def setup(api=None):
    """Sets up and fills test directory for serving.

    Using different filetypes to see how they are dealt with.
    The tempoary directory will clean itself up.
    """
    global tmp_dir_object

    tmp_dir_object = tempfile.TemporaryDirectory()

    dir_name = tmp_dir_object.name

    dir_a = os.path.join(dir_name, "a")
    os.mkdir(dir_a)
    dir_b = os.path.join(dir_name, "b")
    os.mkdir(dir_b)

    # populate directory a with text files
    file_list = [
            ["hi.txt", """Hi World!"""],
            ["hi.html", """<strong>Hi World!</strong>"""],
            ["hello.html", """
                <img src='/static/b/smile.png'</img>
                pop-up
                <script src='/static/a/hi.js'></script>"""],
            ["hi.js", """alert('Hi World')""" ]
    ]

    for f in file_list:
        with open(os.path.join(dir_a, f[0]), mode="wt") as fo:
            fo.write(f[1])

    # populate directory b with binary file
    image = b'\x89PNG\r\n\x1a\n\x00\x00\x00\rIHDR\x00\x00\x00\n\x00\x00\x00\n\x08\x02\x00\x00\x00\x02PX\xea\x00\x00\x006IDAT\x18\xd3c\xfc\xff\xff?\x03n\xc0\xc4\x80\x170100022222\xc2\x85\x90\xb9\x04t3\x92`7\xb2\x15D\xeb\xc6\xe34\xa8n4c\xe1F\x120\x1c\x00\xc6z\x12\x1c\x8cT\xf2\x1e\x00\x00\x00\x00IEND\xaeB`\x82'

    with open(os.path.join(dir_b, "smile.png"), mode="wb") as fo:
            fo.write(image)


@hug.static('/static')
def my_static_dirs():
    """Returns static directory names to be served."""
    global tmp_dir_object
    if tmp_dir_object == None:
        setup()
    return(tmp_dir_object.name,)



"""tests/test_authentication.py.

Tests hugs built-in authentication helper methods

Copyright (C) 2016 Timothy Edmund Crosley

Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated
documentation files (the "Software"), to deal in the Software without restriction, including without limitation
the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and
to permit persons to whom the Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or
substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.

"""
from base64 import b64encode

from falcon import HTTPUnauthorized
import hug


api = hug.API(__name__)


def test_basic_auth():
    """Test to ensure hug provides basic_auth handler works as expected"""

    @hug.get(requires=hug.authentication.basic(hug.authentication.verify('Tim', 'Custom password')))
    def hello_world():
        return 'Hello world!'

    assert '401' in hug.test.get(api, 'hello_world').status
    assert '401' in hug.test.get(api, 'hello_world', headers={'Authorization': 'Not correctly formed'}).status
    assert '401' in hug.test.get(api, 'hello_world', headers={'Authorization': 'Nospaces'}).status
    assert '401' in hug.test.get(api, 'hello_world', headers={'Authorization': 'Basic VXNlcjE6bXlwYXNzd29yZA'}).status

    token = b64encode('{0}:{1}'.format('Tim', 'Custom password').encode('utf8')).decode('utf8')
    assert hug.test.get(api, 'hello_world', headers={'Authorization': 'Basic {0}'.format(token)}).data == 'Hello world!'

    token = b'Basic ' + b64encode('{0}:{1}'.format('Tim', 'Custom password').encode('utf8'))
    assert hug.test.get(api, 'hello_world', headers={'Authorization': token}).data == 'Hello world!'

    token = b'Basic ' + b64encode('{0}:{1}'.format('Tim', 'Wrong password').encode('utf8'))
    assert '401' in hug.test.get(api, 'hello_world', headers={'Authorization': token}).status

    custom_context = dict(custom='context', username='Tim', password='Custom password')

    @hug.context_factory()
    def create_test_context(*args, **kwargs):
        return custom_context

    @hug.delete_context()
    def delete_custom_context(context, exception=None, errors=None, lacks_requirement=None):
        assert context == custom_context
        assert not errors
        context['exception'] = exception

    @hug.authentication.basic
    def context_basic_authentication(username, password, context):
        assert context == custom_context
        if username == context['username'] and password == context['password']:
            return True

    @hug.get(requires=context_basic_authentication)
    def hello_context():
        return 'context!'

    assert '401' in hug.test.get(api, 'hello_context').status
    assert isinstance(custom_context['exception'], HTTPUnauthorized)
    del custom_context['exception']
    assert '401' in hug.test.get(api, 'hello_context', headers={'Authorization': 'Not correctly formed'}).status
    assert isinstance(custom_context['exception'], HTTPUnauthorized)
    del custom_context['exception']
    assert '401' in hug.test.get(api, 'hello_context', headers={'Authorization': 'Nospaces'}).status
    assert isinstance(custom_context['exception'], HTTPUnauthorized)
    del custom_context['exception']
    assert '401' in hug.test.get(api, 'hello_context', headers={'Authorization': 'Basic VXNlcjE6bXlwYXNzd29yZA'}).status
    assert isinstance(custom_context['exception'], HTTPUnauthorized)
    del custom_context['exception']

    token = b64encode('{0}:{1}'.format('Tim', 'Custom password').encode('utf8')).decode('utf8')
    assert hug.test.get(api, 'hello_context', headers={'Authorization': 'Basic {0}'.format(token)}).data == 'context!'
    assert not custom_context['exception']
    del custom_context['exception']
    token = b'Basic ' + b64encode('{0}:{1}'.format('Tim', 'Custom password').encode('utf8'))
    assert hug.test.get(api, 'hello_context', headers={'Authorization': token}).data == 'context!'
    assert not custom_context['exception']
    del custom_context['exception']
    token = b'Basic ' + b64encode('{0}:{1}'.format('Tim', 'Wrong password').encode('utf8'))
    assert '401' in hug.test.get(api, 'hello_context', headers={'Authorization': token}).status
    assert isinstance(custom_context['exception'], HTTPUnauthorized)
    del custom_context['exception']


def test_api_key():
    """Test the included api_key based header to ensure it works as expected to allow X-Api-Key based authentication"""

    @hug.authentication.api_key
    def api_key_authentication(api_key):
        if api_key == 'Bacon':
            return 'Timothy'

    @hug.get(requires=api_key_authentication)
    def hello_world():
        return 'Hello world!'

    assert hug.test.get(api, 'hello_world', headers={'X-Api-Key': 'Bacon'}).data == 'Hello world!'
    assert '401' in hug.test.get(api, 'hello_world').status
    assert '401' in hug.test.get(api, 'hello_world', headers={'X-Api-Key': 'Invalid'}).status

    custom_context = dict(custom='context')

    @hug.context_factory()
    def create_test_context(*args, **kwargs):
        return custom_context

    @hug.delete_context()
    def delete_custom_context(context, exception=None, errors=None, lacks_requirement=None):
        assert context == custom_context
        assert not errors
        context['exception'] = exception

    @hug.authentication.api_key
    def context_api_key_authentication(api_key, context):
        assert context == custom_context
        if api_key == 'Bacon':
            return 'Timothy'

    @hug.get(requires=context_api_key_authentication)
    def hello_context_world():
        return 'Hello context world!'

    assert hug.test.get(api, 'hello_context_world', headers={'X-Api-Key': 'Bacon'}).data == 'Hello context world!'
    assert not custom_context['exception']
    del custom_context['exception']
    assert '401' in hug.test.get(api, 'hello_context_world').status
    assert isinstance(custom_context['exception'], HTTPUnauthorized)
    del custom_context['exception']
    assert '401' in hug.test.get(api, 'hello_context_world', headers={'X-Api-Key': 'Invalid'}).status
    assert isinstance(custom_context['exception'], HTTPUnauthorized)
    del custom_context['exception']


def test_token_auth():
    """Test JSON Web Token"""
    #generated with jwt.encode({'user': 'Timothy','data':'my data'}, 'super-secret-key-please-change', algorithm='HS256')
    precomptoken = 'eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9.eyJkYXRhIjoibXkgZGF0YSIsInVzZXIiOiJUaW1vdGh5In0.' \
                   '8QqzQMJUTq0Dq7vHlnDjdoCKFPDAlvxGCpc_8XF41nI'

    @hug.authentication.token
    def token_authentication(token):
        if token == precomptoken:
            return 'Timothy'

    @hug.get(requires=token_authentication)
    def hello_world():
        return 'Hello World!'

    assert hug.test.get(api, 'hello_world', headers={'Authorization': precomptoken}).data == 'Hello World!'
    assert '401' in hug.test.get(api, 'hello_world').status
    assert '401' in hug.test.get(api, 'hello_world', headers={'Authorization': 'eyJhbGci'}).status

    custom_context = dict(custom='context')

    @hug.context_factory()
    def create_test_context(*args, **kwargs):
        return custom_context

    @hug.delete_context()
    def delete_custom_context(context, exception=None, errors=None, lacks_requirement=None):
        assert context == custom_context
        assert not errors
        context['exception'] = exception

    @hug.authentication.token
    def context_token_authentication(token, context):
        assert context == custom_context
        if token == precomptoken:
            return 'Timothy'

    @hug.get(requires=context_token_authentication)
    def hello_context_world():
        return 'Hello context!'

    assert hug.test.get(api, 'hello_context_world', headers={'Authorization': precomptoken}).data == 'Hello context!'
    assert not custom_context['exception']
    del custom_context['exception']
    assert '401' in hug.test.get(api, 'hello_context_world').status
    assert isinstance(custom_context['exception'], HTTPUnauthorized)
    del custom_context['exception']
    assert '401' in hug.test.get(api, 'hello_context_world', headers={'Authorization': 'eyJhbGci'}).status
    assert isinstance(custom_context['exception'], HTTPUnauthorized)
    del custom_context['exception']


def test_documentation_carry_over():
    """Test to ensure documentation correctly carries over - to address issue #252"""
    authentication = hug.authentication.basic(hug.authentication.verify('User1', 'mypassword'))
    assert authentication.__doc__ == 'Basic HTTP Authentication'


def test_missing_authenticator_docstring():

    @hug.authentication.authenticator
    def custom_authenticator(*args, **kwargs):
        return None

    authentication = custom_authenticator(None)

    @hug.get(requires=authentication)
    def hello_world():
        return 'Hello World!'

    hug.test.get(api, 'hello_world')



"""tests/test_decorators.py.

Tests that class based hug routes interact as expected

Copyright (C) 2016 Timothy Edmund Crosley

Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated
documentation files (the "Software"), to deal in the Software without restriction, including without limitation
the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and
to permit persons to whom the Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or
substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.

"""
import hug
from hug.routing import CLIRouter, ExceptionRouter, NotFoundRouter, SinkRouter, StaticRouter, URLRouter

api = hug.API(__name__)


def test_simple_class_based_view():
    """Test creating class based routers"""
    @hug.object.urls('/endpoint', requires=())
    class MyClass(object):

        @hug.object.get()
        def my_method(self):
            return 'hi there!'

        @hug.object.post()
        def my_method_two(self):
            return 'bye'

    assert hug.test.get(api, 'endpoint').data == 'hi there!'
    assert hug.test.post(api, 'endpoint').data == 'bye'


def test_url_inheritance():
    """Test creating class based routers"""
    @hug.object.urls('/endpoint', requires=(), versions=1)
    class MyClass(object):

        @hug.object.urls('inherits_base')
        def my_method(self):
            return 'hi there!'

        @hug.object.urls('/ignores_base')
        def my_method_two(self):
            return 'bye'

        @hug.object.urls('ignore_version', versions=None)
        def my_method_three(self):
            return 'what version?'

    assert hug.test.get(api, '/v1/endpoint/inherits_base').data == 'hi there!'
    assert hug.test.post(api, '/v1/ignores_base').data == 'bye'
    assert hug.test.post(api, '/v2/ignores_base').data != 'bye'
    assert hug.test.get(api, '/endpoint/ignore_version').data == 'what version?'


def test_simple_class_based_method_view():
    """Test creating class based routers using method mappings"""
    @hug.object.http_methods()
    class EndPoint(object):

        def get(self):
            return 'hi there!'

        def post(self):
            return 'bye'

    assert hug.test.get(api, 'endpoint').data == 'hi there!'
    assert hug.test.post(api, 'endpoint').data == 'bye'


def test_routing_class_based_method_view_with_sub_routing():
    """Test creating class based routers using method mappings, then overriding url on sub method"""
    @hug.object.http_methods()
    class EndPoint(object):

        def get(self):
            return 'hi there!'

        @hug.object.urls('/home/')
        def post(self):
            return 'bye'

    assert hug.test.get(api, 'endpoint').data == 'hi there!'
    assert hug.test.post(api, 'home').data == 'bye'


def test_routing_class_with_cli_commands():
    """Basic operation test"""
    @hug.object(name='git', version='1.0.0')
    class GIT(object):
        """An example of command like calls via an Object"""

        @hug.object.cli
        def push(self, branch='master'):
            return 'Pushing {}'.format(branch)

        @hug.object.cli
        def pull(self, branch='master'):
            return 'Pulling {}'.format(branch)

    assert 'token' in hug.test.cli(GIT.push, branch='token')
    assert 'another token' in hug.test.cli(GIT.pull, branch='another token')


def test_routing_class_based_method_view_with_cli_routing():
    """Test creating class based routers using method mappings exposing cli endpoints"""
    @hug.object.http_methods()
    class EndPoint(object):

        @hug.object.cli
        def get(self):
            return 'hi there!'

        def post(self):
            return 'bye'

    assert hug.test.get(api, 'endpoint').data == 'hi there!'
    assert hug.test.post(api, 'endpoint').data == 'bye'
    assert hug.test.cli(EndPoint.get) == 'hi there!'


def test_routing_instance():
    """Test to ensure its possible to route a class after it is instanciated"""
    class EndPoint(object):

        @hug.object
        def one(self):
            return 'one'

        @hug.object
        def two(self):
            return 2

    hug.object.get()(EndPoint())
    assert hug.test.get(api, 'one').data == 'one'
    assert hug.test.get(api, 'two').data == 2


class TestAPIRouter(object):
    """Test to ensure the API router enables easily reusing all other routing types while routing to an API"""
    router = hug.route.API(__name__)

    def test_route_url(self):
        """Test to ensure you can dynamically create a URL route attached to a hug API"""
        assert self.router.urls('/hi/').route == URLRouter('/hi/', api=api).route

    def test_route_http(self):
        """Test to ensure you can dynamically create an HTTP route attached to a hug API"""
        assert self.router.http('/hi/').route == URLRouter('/hi/', api=api).route

    def test_method_routes(self):
        """Test to ensure you can dynamically create an HTTP route attached to a hug API"""
        for method in hug.HTTP_METHODS:
            assert getattr(self.router, method.lower())('/hi/').route['accept'] == (method, )

        assert self.router.get_post('/hi/').route['accept'] == ('GET', 'POST')
        assert self.router.put_post('/hi/').route['accept'] == ('PUT', 'POST')

    def test_not_found(self):
        """Test to ensure you can dynamically create a Not Found route attached to a hug API"""
        assert self.router.not_found().route == NotFoundRouter(api=api).route

    def test_static(self):
        """Test to ensure you can dynamically create a static route attached to a hug API"""
        assert self.router.static().route == StaticRouter(api=api).route

    def test_sink(self):
        """Test to ensure you can dynamically create a sink route attached to a hug API"""
        assert self.router.sink().route == SinkRouter(api=api).route

    def test_exception(self):
        """Test to ensure you can dynamically create an Exception route attached to a hug API"""
        assert self.router.exception().route == ExceptionRouter(api=api).route

    def test_cli(self):
        """Test to ensure you can dynamically create a CLI route attached to a hug API"""
        assert self.router.cli().route == CLIRouter(api=api).route

    def test_object(self):
        """Test to ensure it's possible to route objects through a specified API instance"""
        assert self.router.object().route == hug.route.Object(api=api).route



# pylint: disable=C0326
""" Base setup script """

from setuptools import setup

setup(
    name = "app-name",
    version = "0.0.1",
    description = "App Description",
    url = "https://github.com/CMoncur/nginx-gunicorn-hug",
    author = "Cody Moncur",
    author_email = "cmoncur@gmail.com",
    classifiers = [
        #   3 - Alpha
        #   4 - Beta
        #   5 - Production/Stable
        "Development Status :: 3 - Alpha",
        "Programming Language :: Python :: 3.6"
    ],
    packages = [],

    # Entry Point
    entry_points = {
        "console_scripts": []
    },

    # Core Dependencies
    install_requires = [
        "hug"
    ],

    # Dev/Test Dependencies
    extras_require = {
        "dev": [],
        "test": [],
    },

    # Scripts
    scripts = []
)



import falcon


class Resource(object):

    def on_get(self, req, resp):
        resp.status = falcon.HTTP_200
        resp.content_type = 'text/plain'
        resp.body = 'Hello, world!'


app = falcon.API()
app.add_route('/text', Resource())



from sqlalchemy.sql.schema import Column
from sqlalchemy.sql.sqltypes import Integer, String

from demo.base import Base


class TestModel(Base):
    __tablename__ = 'test_model'
    id = Column(Integer, primary_key=True)
    name = Column(String)


class TestUser(Base):
    __tablename__ = 'test_user'
    id = Column(Integer, primary_key=True)
    username = Column(String)
    password = Column(String)  # do not store plain password in the database, hash it, see porridge for example



import hug
import happy_birthday
from falcon import HTTP_400, HTTP_404, HTTP_200

def tests_happy_birthday():
    response = hug.test.get(happy_birthday, 'happy_birthday', {'name': 'Timothy', 'age': 25})
    assert response.status == HTTP_200
    assert response.data is not None

def tests_season_greetings():
    response = hug.test.get(happy_birthday, 'greet/Christmas')
    assert response.status == HTTP_200
    assert response.data is not None
    assert str(response.data) == "Merry Christmas!"
    response = hug.test.get(happy_birthday, 'greet/holidays')
    assert str(response.data) == "Happy holidays!"
